"""Initializing module India """
# @coding: utf-8
# @author: Rishabh Batra
# @email: rishabhbatra1002@gmail.com

# Module imports
from . import states, cities
from .states import STATES, UNION_TERRITORIES, STATES_AND_TERRITORIES
from .cities import CITIES

__appname__ = 'pyindia'
__version__ = '1.1.1'
